@@ -179,10 +179,10 @@ Rcpp::List rangerCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
       }
     } else {
       // Set class weights
-      if (treetype == TREE_CLASSIFICATION) {
+      if (treetype == TREE_CLASSIFICATION && !class_weights.empty()) {
         auto& temp = dynamic_cast<ForestClassification&>(*forest);
         temp.setClassWeights(class_weights);
-      } else if (treetype == TREE_PROBABILITY) {
+      } else if (treetype == TREE_PROBABILITY && !class_weights.empty()) {
         auto& temp = dynamic_cast<ForestProbability&>(*forest);
         temp.setClassWeights(class_weights);
       }
@@ -218,11 +218,6 @@ Rcpp::List rangerCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
       auto& temp = dynamic_cast<ForestSurvival&>(*forest);
       result.push_back(temp.getUniqueTimepoints(), "unique.death.times");
     }
-    if (!verbose) {
-      std::stringstream temp;
-      temp << verbose_out->rdbuf();
-      result.push_back(temp.str(), "log");
-    }
     if (!prediction_mode) {
       result.push_back(forest->getMtry(), "mtry");
       result.push_back(forest->getMinNodeSize(), "min.node.size");
@@ -267,6 +262,10 @@ Rcpp::List rangerCpp(uint treetype, std::string dependent_variable_name, Rcpp::N
       }
       result.push_back(forest_object, "forest");
     }
+    
+    if (!verbose) {
+      delete verbose_out;
+    }
   } catch (std::exception& e) {
     if (strcmp(e.what(), "User interrupt.") != 0) {
       Rcpp::Rcerr << "Error: " << e.what() << " Ranger will EXIT now." << std::endl;
